import asyncio
from typing import Callable

from async_lru import alru_cache


async def test_cache_invalidate(check_lru: Callable[..., None]) -> None:
    @alru_cache()
    async def coro(val: int) -> int:
        return val

    inputs = [1, 2, 3]

    coro.cache_invalidate(1)
    coro.cache_invalidate(2)
    coro.cache_invalidate(3)

    coros = [coro(v) for v in inputs]

    ret = await asyncio.gather(*coros)

    assert ret == inputs
    check_lru(coro, hits=0, misses=3, cache=3, tasks=0)

    coro.cache_invalidate(1)
    check_lru(coro, hits=0, misses=3, cache=2, tasks=0)
    coro.cache_invalidate(2)
    check_lru(coro, hits=0, misses=3, cache=1, tasks=0)
    coro.cache_invalidate(3)
    check_lru(coro, hits=0, misses=3, cache=0, tasks=0)

    inputs = [1, 2, 3]
    coros = [coro(v) for v in inputs]

    ret = await asyncio.gather(*coros)

    assert ret == inputs
    check_lru(coro, hits=0, misses=6, cache=3, tasks=0)


async def test_cache_invalidate_multiple_args(check_lru: Callable[..., None]) -> None:
    @alru_cache()
    async def coro(*args: int) -> int:
        return len(args)

    for i, size in enumerate(range(10)):
        args = tuple(range(size))
        ret = await coro(*args)
        assert ret == size
        check_lru(coro, hits=0, misses=i + 1, cache=1, tasks=0)
        coro.cache_invalidate(*args)
        check_lru(coro, hits=0, misses=i + 1, cache=0, tasks=0)

    for size in range(10):
        args = tuple(range(size))
        ret = await coro(*args)
        assert ret == size
    check_lru(coro, hits=0, misses=20, cache=10, tasks=0)


async def test_cache_invalidate_multiple_args_different_order(
    check_lru: Callable[..., None]
) -> None:
    @alru_cache()
    async def coro(*args: int) -> int:
        return len(args)

    for i, size in enumerate(range(2, 10)):
        args = tuple(range(size))
        rev_args = tuple(reversed(args))
        ret = await coro(*args)
        assert ret == size
        check_lru(coro, hits=0, misses=2 * i + 1, cache=i + 1, tasks=0)
        ret = await coro(*rev_args)
        # The reversed args should be a miss
        check_lru(coro, hits=0, misses=2 * i + 2, cache=i + 2, tasks=0)
        coro.cache_invalidate(*rev_args)
        # The reversed args should be invalidated
        check_lru(coro, hits=0, misses=2 * i + 2, cache=i + 1, tasks=0)

    for i, size in enumerate(range(2, 10)):
        args = tuple(range(size))
        ret = await coro(*args)
        assert ret == size
        check_lru(coro, hits=i + 1, misses=16, cache=8, tasks=0)



import asyncio
from functools import partial
from unittest import mock

import pytest

from async_lru import _LRUCacheWrapper


async def test_done_callback_cancelled() -> None:
    wrapped = _LRUCacheWrapper(mock.ANY, None, False, None)
    loop = asyncio.get_running_loop()
    task = loop.create_future()
    fut = loop.create_future()

    key = 1

    task.add_done_callback(partial(wrapped._task_done_callback, fut, key))
    wrapped._LRUCacheWrapper__tasks.add(task)  # type: ignore[attr-defined]

    task.cancel()

    await asyncio.sleep(0)

    assert fut.cancelled()


async def test_done_callback_exception() -> None:
    wrapped = _LRUCacheWrapper(mock.ANY, None, False, None)
    loop = asyncio.get_running_loop()
    task = loop.create_future()
    fut = loop.create_future()

    key = 1

    task.add_done_callback(partial(wrapped._task_done_callback, fut, key))
    wrapped._LRUCacheWrapper__tasks.add(task)  # type: ignore[attr-defined]

    exc = ZeroDivisionError()

    task.set_exception(exc)

    await asyncio.sleep(0)

    with pytest.raises(ZeroDivisionError):
        await fut

    with pytest.raises(ZeroDivisionError):
        fut.result()

    assert fut.exception() is exc


async def test_done_callback() -> None:
    wrapped = _LRUCacheWrapper(mock.ANY, None, False, None)
    loop = asyncio.get_running_loop()
    task = loop.create_future()

    key = 1
    fut = loop.create_future()

    task.add_done_callback(partial(wrapped._task_done_callback, fut, key))
    wrapped._LRUCacheWrapper__tasks.add(task)  # type: ignore[attr-defined]

    task.set_result(1)

    await asyncio.sleep(0)

    assert fut.result() == 1


async def test_cache_invalidate_typed() -> None:
    wrapped = _LRUCacheWrapper(mock.AsyncMock(return_value=1), None, True, None)

    from_cache = wrapped.cache_invalidate(1, a=1)

    assert not from_cache

    await wrapped(1, a=1)

    from_cache = wrapped.cache_invalidate(1, a=1)

    assert from_cache

    assert wrapped.cache_info().currsize == 0

    from_cache = wrapped.cache_invalidate(1.0, a=1)
    assert not from_cache

    assert wrapped.cache_info().currsize == 0
    await wrapped(1.0, a=1)
    assert wrapped.cache_info().currsize == 1
    from_cache = wrapped.cache_invalidate(1.0, a=1)
    assert from_cache


async def test_cache_invalidate_not_typed() -> None:
    wrapped = _LRUCacheWrapper(mock.AsyncMock(return_value=1), None, False, None)

    from_cache = wrapped.cache_invalidate(1, a=1)
    assert not from_cache

    await wrapped(1, a=1)
    assert wrapped.cache_info().currsize == 1

    from_cache = wrapped.cache_invalidate(1, a=1)
    assert from_cache
    assert wrapped.cache_info().currsize == 0

    await wrapped(1, a=1)
    assert wrapped.cache_info().currsize == 1

    from_cache = wrapped.cache_invalidate(1.0, a=1)
    assert from_cache
    assert wrapped.cache_info().currsize == 0


async def test_cache_clear() -> None:
    wrapped = _LRUCacheWrapper(mock.AsyncMock(return_value=1), None, True, None)

    await wrapped(123)

    assert wrapped.cache_info().hits == 0
    assert wrapped.cache_info().misses == 1
    assert wrapped.cache_info().currsize == 1
    assert wrapped.cache_parameters()["tasks"] == 0

    await wrapped(123)
    assert wrapped.cache_info().hits == 1
    assert wrapped.cache_info().misses == 1
    assert wrapped.cache_info().currsize == 1
    assert wrapped.cache_parameters()["tasks"] == 0

    wrapped.cache_clear()

    assert wrapped.cache_info().hits == 0
    assert wrapped.cache_info().misses == 0
    assert wrapped.cache_info().currsize == 0
    assert wrapped.cache_parameters()["tasks"] == 0


def test_cache_info() -> None:
    wrapped = _LRUCacheWrapper(mock.ANY, 3, True, None)

    assert (0, 0, 3, 0) == wrapped.cache_info()

    wrapped._LRUCacheWrapper__cache[1] = 1  # type: ignore[attr-defined]

    assert (0, 0, 3, 1) == wrapped.cache_info()

    wrapped._LRUCacheWrapper__hits = 2  # type: ignore[attr-defined]
    wrapped._LRUCacheWrapper__misses = 3  # type: ignore[attr-defined]
    wrapped._LRUCacheWrapper__cache[2] = 2  # type: ignore[attr-defined]

    assert (2, 3, 3, 2) == wrapped.cache_info()


async def test_cache_hit() -> None:
    wrapped = _LRUCacheWrapper(mock.AsyncMock(return_value=1), None, True, None)
    await wrapped(1)
    assert wrapped.cache_info().hits == 0
    assert wrapped.cache_info().misses == 1
    await wrapped(1)
    assert wrapped.cache_info().hits == 1
    assert wrapped.cache_info().misses == 1
    await wrapped(1)
    assert wrapped.cache_info().hits == 2
    assert wrapped.cache_info().misses == 1


async def test_cache_miss() -> None:
    wrapped = _LRUCacheWrapper(mock.AsyncMock(return_value=1), None, True, None)
    await wrapped(1)
    assert wrapped.cache_info().hits == 0
    assert wrapped.cache_info().misses == 1
    await wrapped(2)
    assert wrapped.cache_info().hits == 0
    assert wrapped.cache_info().misses == 2
    await wrapped(3)
    assert wrapped.cache_info().hits == 0
    assert wrapped.cache_info().misses == 3


async def test_forbid_call_closed() -> None:
    wrapped = _LRUCacheWrapper(mock.AsyncMock(return_value=1), None, True, None)
    wrapped._LRUCacheWrapper__closed = True  # type: ignore[attr-defined]
    with pytest.raises(RuntimeError):
        await wrapped(123)



import asyncio
from typing import Callable

import pytest

from async_lru import alru_cache


async def test_cache_close(check_lru: Callable[..., None]) -> None:
    @alru_cache()
    async def coro(val: int) -> int:
        await asyncio.sleep(0.2)

        return val

    assert not coro.cache_parameters()["closed"]

    inputs = [1, 2, 3, 4, 5]

    coros = [coro(v) for v in inputs]

    gather = asyncio.gather(*coros)

    await asyncio.sleep(0.1)

    check_lru(coro, hits=0, misses=5, cache=5, tasks=5)

    close = coro.cache_close()

    check_lru(coro, hits=0, misses=5, cache=5, tasks=5)

    await close

    check_lru(coro, hits=0, misses=5, cache=5, tasks=0)
    assert coro.cache_parameters()["closed"]

    with pytest.raises(asyncio.CancelledError):
        await gather

    check_lru(coro, hits=0, misses=5, cache=5, tasks=0)
    assert coro.cache_parameters()["closed"]

    # double call is no-op
    await coro.cache_close()



import asyncio
from functools import partial, partialmethod
from typing import Callable

from async_lru import alru_cache


async def test_partialmethod_basic(check_lru: Callable[..., None]) -> None:
    class Obj:
        async def _coro(self, val: int) -> int:
            return val

        coro = alru_cache(partialmethod(_coro, 2))

    obj = Obj()

    coros = [obj.coro() for _ in range(5)]

    check_lru(obj.coro, hits=0, misses=0, cache=0, tasks=0)

    ret = await asyncio.gather(*coros)

    check_lru(obj.coro, hits=4, misses=1, cache=1, tasks=0)

    assert ret == [2, 2, 2, 2, 2]


async def test_partialmethod_partial(check_lru: Callable[..., None]) -> None:
    class Obj:
        def __init__(self) -> None:
            self.coro = alru_cache(partial(self._coro, 2))

        async def __coro(self, val1: int, val2: int) -> int:
            return val1 + val2

        _coro = partialmethod(__coro, 1)

    obj = Obj()

    coros = [obj.coro() for _ in range(5)]

    check_lru(obj.coro, hits=0, misses=0, cache=0, tasks=0)

    ret = await asyncio.gather(*coros)

    check_lru(obj.coro, hits=4, misses=1, cache=1, tasks=0)

    assert ret == [3, 3, 3, 3, 3]



import asyncio
from typing import Callable

from async_lru import alru_cache


async def test_ttl_infinite_cache(check_lru: Callable[..., None]) -> None:
    @alru_cache(maxsize=None, ttl=0.1)
    async def coro(val: int) -> int:
        return val

    assert await coro(1) == 1
    check_lru(coro, hits=0, misses=1, cache=1, tasks=0, maxsize=None)
    await asyncio.sleep(0.0)
    assert await coro(1) == 1
    check_lru(coro, hits=1, misses=1, cache=1, tasks=0, maxsize=None)

    await asyncio.sleep(0.2)
    # cache is clear after ttl expires
    check_lru(coro, hits=1, misses=1, cache=0, tasks=0, maxsize=None)
    assert await coro(1) == 1
    check_lru(coro, hits=1, misses=2, cache=1, tasks=0, maxsize=None)


async def test_ttl_limited_cache(check_lru: Callable[..., None]) -> None:
    @alru_cache(maxsize=1, ttl=0.1)
    async def coro(val: int) -> int:
        return val

    assert await coro(1) == 1
    check_lru(coro, hits=0, misses=1, cache=1, tasks=0, maxsize=1)

    assert await coro(2) == 2
    check_lru(coro, hits=0, misses=2, cache=1, tasks=0, maxsize=1)

    await asyncio.sleep(0)
    assert await coro(2) == 2
    check_lru(coro, hits=1, misses=2, cache=1, tasks=0, maxsize=1)

    assert await coro(1) == 1
    check_lru(coro, hits=1, misses=3, cache=1, tasks=0, maxsize=1)


async def test_ttl_with_explicit_invalidation(check_lru: Callable[..., None]) -> None:
    @alru_cache(maxsize=None, ttl=0.2)
    async def coro(val: int) -> int:
        return val

    assert await coro(1) == 1
    check_lru(coro, hits=0, misses=1, cache=1, tasks=0, maxsize=None)
    coro.cache_invalidate(1)
    check_lru(coro, hits=0, misses=1, cache=0, tasks=0, maxsize=None)
    await asyncio.sleep(0.1)
    assert await coro(1) == 1
    check_lru(coro, hits=0, misses=2, cache=1, tasks=0, maxsize=None)

    await asyncio.sleep(0.1)
    # cache is not cleared after ttl expires because invalidate also should clear
    # the invalidation by timeout
    check_lru(coro, hits=0, misses=2, cache=1, tasks=0, maxsize=None)


async def test_ttl_concurrent() -> None:
    @alru_cache(maxsize=1, ttl=1)
    async def coro(val: int) -> int:
        return val

    results = await asyncio.gather(*(coro(i) for i in range(2)))
    assert results == list(range(2))



import asyncio
from typing import Callable

from async_lru import alru_cache


async def test_cache_info(check_lru: Callable[..., None]) -> None:
    @alru_cache(maxsize=4)
    async def coro(val: int) -> int:
        return val

    inputs = [1, 2, 3]
    coros = [coro(v) for v in inputs]
    ret = await asyncio.gather(*coros)
    assert ret == inputs
    check_lru(coro, hits=0, misses=3, cache=3, tasks=0, maxsize=4)

    coro.cache_clear()

    check_lru(coro, hits=0, misses=0, cache=0, tasks=0, maxsize=4)

    inputs = [1, 1, 1]
    coros = [coro(v) for v in inputs]
    ret = await asyncio.gather(*coros)
    assert ret == inputs
    check_lru(coro, hits=2, misses=1, cache=1, tasks=0, maxsize=4)

    coro.cache_clear()

    check_lru(coro, hits=0, misses=0, cache=0, tasks=0, maxsize=4)

    inputs = [1, 2, 3, 4] * 2
    coros = [coro(v) for v in inputs]
    ret = await asyncio.gather(*coros)
    assert ret == inputs
    check_lru(coro, hits=4, misses=4, cache=4, tasks=0, maxsize=4)



import asyncio
from functools import _CacheInfo, partial
from typing import Callable

import pytest

from async_lru import _CacheParameters, alru_cache


def test_alru_cache_not_callable() -> None:
    with pytest.raises(NotImplementedError):
        alru_cache("foo")  # type: ignore[call-overload]


def test_alru_cache_not_coroutine() -> None:
    with pytest.raises(RuntimeError):

        @alru_cache  # type: ignore[arg-type]
        def not_coro(val: int) -> int:
            return val


async def test_alru_cache_deco(check_lru: Callable[..., None]) -> None:
    @alru_cache
    async def coro() -> None:
        pass

    assert asyncio.iscoroutinefunction(coro)

    check_lru(coro, hits=0, misses=0, cache=0, tasks=0)

    awaitable = coro()
    assert asyncio.iscoroutine(awaitable)
    await awaitable


async def test_alru_cache_deco_called(check_lru: Callable[..., None]) -> None:
    @alru_cache()
    async def coro() -> None:
        pass

    assert asyncio.iscoroutinefunction(coro)

    check_lru(coro, hits=0, misses=0, cache=0, tasks=0)

    awaitable = coro()
    assert asyncio.iscoroutine(awaitable)
    await awaitable


async def test_alru_cache_fn_called(check_lru: Callable[..., None]) -> None:
    async def coro() -> None:
        pass

    coro_wrapped = alru_cache(coro)

    assert asyncio.iscoroutinefunction(coro_wrapped)

    check_lru(coro_wrapped, hits=0, misses=0, cache=0, tasks=0)

    awaitable = coro_wrapped()
    assert asyncio.iscoroutine(awaitable)
    await awaitable


async def test_alru_cache_partial() -> None:
    async def coro(val: int) -> int:
        return val

    coro_wrapped1 = alru_cache(coro)

    assert await coro_wrapped1(1) == 1

    coro_wrapped2 = alru_cache(partial(coro, 2))

    assert await coro_wrapped2() == 2


async def test_alru_cache_await_same_result_async(
    check_lru: Callable[..., None]
) -> None:
    calls = 0
    val = object()

    @alru_cache()
    async def coro() -> object:
        nonlocal calls
        calls += 1

        return val

    coros = [coro() for _ in range(100)]
    ret = await asyncio.gather(*coros)
    expected = [val] * 100
    assert ret == expected
    check_lru(coro, hits=99, misses=1, cache=1, tasks=0)

    assert calls == 1
    assert await coro() is val
    check_lru(coro, hits=100, misses=1, cache=1, tasks=0)


async def test_alru_cache_await_same_result_coroutine(
    check_lru: Callable[..., None]
) -> None:
    calls = 0
    val = object()

    @alru_cache()
    async def coro() -> object:
        nonlocal calls
        calls += 1

        return val

    coros = [coro() for _ in range(100)]
    ret = await asyncio.gather(*coros)
    expected = [val] * 100
    assert ret == expected
    check_lru(coro, hits=99, misses=1, cache=1, tasks=0)

    assert calls == 1
    assert await coro() is val
    check_lru(coro, hits=100, misses=1, cache=1, tasks=0)


async def test_alru_cache_dict_not_shared(check_lru: Callable[..., None]) -> None:
    async def coro(val: int) -> int:
        return val

    coro1 = alru_cache()(coro)
    coro2 = alru_cache()(coro)

    ret1 = await coro1(1)
    check_lru(coro1, hits=0, misses=1, cache=1, tasks=0)

    ret2 = await coro2(1)
    check_lru(coro2, hits=0, misses=1, cache=1, tasks=0)

    assert ret1 == ret2

    assert (
        coro1._LRUCacheWrapper__cache[1].fut.result()  # type: ignore[attr-defined]
        == coro2._LRUCacheWrapper__cache[1].fut.result()  # type: ignore[attr-defined]
    )
    assert coro1._LRUCacheWrapper__cache != coro2._LRUCacheWrapper__cache  # type: ignore[attr-defined]
    assert coro1._LRUCacheWrapper__cache.keys() == coro2._LRUCacheWrapper__cache.keys()  # type: ignore[attr-defined]
    assert coro1._LRUCacheWrapper__cache is not coro2._LRUCacheWrapper__cache  # type: ignore[attr-defined]


async def test_alru_cache_parameters() -> None:
    @alru_cache
    async def coro(val: int) -> int:
        return val

    assert coro.cache_parameters() == _CacheParameters(
        typed=False,
        maxsize=128,
        tasks=0,
        closed=False,
    )

    await coro(1)
    assert coro.cache_parameters() == _CacheParameters(
        typed=False,
        maxsize=128,
        tasks=0,
        closed=False,
    )


async def test_alru_cache_method() -> None:
    class A:
        def __init__(self, val: int) -> None:
            self.val = val

        @alru_cache
        async def coro(self) -> int:
            return self.val

    a = A(42)
    assert await a.coro() == 42
    assert a.coro.cache_parameters() == _CacheParameters(
        typed=False,
        maxsize=128,
        tasks=0,
        closed=False,
    )


async def test_invalidate_cache_for_method() -> None:
    class A:
        @alru_cache
        async def coro(self, val: int) -> int:
            return val

    a = A()
    assert await a.coro(42) == 42

    assert a.coro.cache_info() == _CacheInfo(0, 1, 128, 1)

    a.coro.cache_invalidate(42)

    assert a.coro.cache_info() == _CacheInfo(0, 1, 128, 0)



from functools import _CacheInfo
from typing import Callable

import pytest

from async_lru import _R, _LRUCacheWrapper


@pytest.fixture
def check_lru() -> Callable[..., None]:
    def _check_lru(
        wrapped: _LRUCacheWrapper[_R],
        *,
        hits: int,
        misses: int,
        cache: int,
        tasks: int,
        maxsize: int = 128
    ) -> None:
        assert wrapped.cache_info() == _CacheInfo(
            hits=hits,
            misses=misses,
            maxsize=maxsize,
            currsize=cache,
        )
        assert wrapped.cache_parameters()["tasks"] == tasks

    return _check_lru



import asyncio
from typing import Callable

import pytest

from async_lru import alru_cache


async def test_alru_exception(check_lru: Callable[..., None]) -> None:
    @alru_cache()
    async def coro(val: int) -> None:
        1 / 0

    inputs = [1, 1, 1]
    coros = [coro(v) for v in inputs]

    ret = await asyncio.gather(*coros, return_exceptions=True)

    check_lru(coro, hits=2, misses=1, cache=1, tasks=0)

    for item in ret:
        assert isinstance(item, ZeroDivisionError)

    with pytest.raises(ZeroDivisionError):
        await coro(1)

    check_lru(coro, hits=2, misses=2, cache=1, tasks=0)



import asyncio
from typing import Callable

from async_lru import alru_cache


async def test_cache_clear(check_lru: Callable[..., None]) -> None:
    @alru_cache()
    async def coro(val: int) -> int:
        return val

    inputs = [1, 2, 3]
    coros = [coro(v) for v in inputs]
    ret = await asyncio.gather(*coros)
    assert ret == inputs
    check_lru(coro, hits=0, misses=3, cache=3, tasks=0)

    coro.cache_clear()

    check_lru(coro, hits=0, misses=0, cache=0, tasks=0)


async def test_cache_clear_pending_task() -> None:
    @alru_cache()
    async def coro() -> str:
        await asyncio.sleep(0.5)
        return "foo"

    t = asyncio.create_task(coro())
    await asyncio.sleep(0)
    assert len(coro._LRUCacheWrapper__tasks) == 1  # type: ignore[attr-defined]
    inner_task = next(iter(coro._LRUCacheWrapper__tasks))  # type: ignore[attr-defined]
    assert not inner_task.done()

    coro.cache_clear()
    await inner_task

    assert await t == "foo"
    assert inner_task.done()


async def test_cache_clear_ttl_callback(check_lru: Callable[..., None]) -> None:
    @alru_cache(ttl=0.5)
    async def coro() -> str:
        return "foo"

    await coro()
    assert len(coro._LRUCacheWrapper__cache) == 1  # type: ignore[attr-defined]
    cache_item = next(iter(coro._LRUCacheWrapper__cache.values()))  # type: ignore[attr-defined]
    assert not cache_item.later_call.cancelled()

    coro.cache_clear()

    assert cache_item.later_call.cancelled()
    await asyncio.sleep(0.5)



import asyncio
from typing import Callable

from async_lru import alru_cache


async def test_alru_cache_removing_lru_keys(check_lru: Callable[..., None]) -> None:
    @alru_cache(maxsize=3)
    async def coro(val: int) -> int:
        return val

    for i, v in enumerate([3, 4, 5]):
        await coro(v)
        check_lru(coro, hits=0, misses=i + 1, cache=i + 1, tasks=0, maxsize=3)

    check_lru(coro, hits=0, misses=3, cache=3, tasks=0, maxsize=3)
    assert list(coro._LRUCacheWrapper__cache) == [3, 4, 5]  # type: ignore[attr-defined]

    for v in [3, 2, 1]:
        await coro(v)
    check_lru(coro, hits=1, misses=5, cache=3, tasks=0, maxsize=3)
    assert list(coro._LRUCacheWrapper__cache) == [3, 2, 1]  # type: ignore[attr-defined]


async def test_alru_cache_removing_lru_keys_with_full_displacement(
    check_lru: Callable[..., None]
) -> None:
    @alru_cache(maxsize=3)
    async def coro(val: int) -> int:
        return val

    for i, v in enumerate([3, 4, 5]):
        await coro(v)
        check_lru(coro, hits=0, misses=i + 1, cache=i + 1, tasks=0, maxsize=3)

    check_lru(coro, hits=0, misses=3, cache=3, tasks=0, maxsize=3)
    assert list(coro._LRUCacheWrapper__cache) == [3, 4, 5]  # type: ignore[attr-defined]

    for v in [1, 2, 3]:
        await coro(v)
    check_lru(coro, hits=0, misses=6, cache=3, tasks=0, maxsize=3)
    assert list(coro._LRUCacheWrapper__cache) == [1, 2, 3]  # type: ignore[attr-defined]


async def test_alru_cache_none_max_size(check_lru: Callable[..., None]) -> None:
    @alru_cache(maxsize=None)
    async def coro(val: int) -> int:
        return val

    inputs = [1, 2, 3, 4] * 2
    coros = [coro(v) for v in inputs]

    ret = await asyncio.gather(*coros)

    check_lru(coro, hits=4, misses=4, cache=4, tasks=0, maxsize=None)
    assert ret == inputs


async def test_alru_cache_zero_max_size(check_lru: Callable[..., None]) -> None:
    @alru_cache(maxsize=0)
    async def coro(val: int) -> int:
        return val

    inputs = [1, 2, 3, 4] * 2
    coros = [coro(v) for v in inputs]

    ret = await asyncio.gather(*coros)

    check_lru(coro, hits=0, misses=8, cache=0, tasks=0, maxsize=0)
    assert ret == inputs
